/***********************************************************************
 * Copyright (c) 2013, Sylvain LAPERCHE                                *
 * All rights reserved.                                                *
 * License: BSD 3-Clause (http://opensource.org/licenses/BSD-3-Clause) *
 ***********************************************************************/
/*
 *    Filename: example.c
 * Description: A simple example to illustrate the use of the slist API.
 *     Created: 2013/08/23 19:47:49
 *      Author: Sylvain Laperche <sylvain.laperche@gmail.com>
 */
#include <stdio.h>
#include <stdlib.h>

#include "slist.h"

typedef struct {
	unsigned to;
	slist_node node;
	unsigned from;
} pair_list;

int main(void)
{
	slist list;
	slist_node* curr;
	slist_node* tmp;
	pair_list*  elt;
	unsigned i;
	/* Initialize the list. */
	slist_init(&list);
	/* Populate the list. */
	for(i = 0; i < 5; ++i) {
		elt = malloc(sizeof *elt);
		if (elt == NULL)
			goto failure;

		elt->from = i;
		elt->to   = i+1;
		slist_add(&list, &(elt->node));
	}
	/* Iteration with `slist_foreach`. */
	puts("List content (before reverse):");
	slist_foreach(&list, curr) {
		elt = slist_elt(curr, pair_list, node);
		printf("from = %u | to = %u\n", elt->from, elt->to);
	}
	/* Reverse the list in-place. */
	puts("\nReverse the list.");
	slist_reverse(&list);
	/* Iteration with `slist_foreach_elt`. */
	puts("\nList content (after reverse):");
	slist_foreach_elt(&list, elt, pair_list, node)
		printf("from = %u | to = %u\n", elt->from, elt->to);
	/*
	 * List destruction with `slist_foreach_elt_safe` and `slist_del`.
	 *
	 * This is the only safe way to free a list. Always use `slist_del` and
	 * `slist_foreach_safe` (or `slist_foreach_elt_safe`) for this purpose.
	 * Otherwise the behavior will be undefined/
	 */
	puts("\nFree the list.");
	slist_foreach_elt_safe(&list, elt, tmp, pair_list, node) {
		printf("Freeing item from = %u | to = %u\n", elt->from, elt->to);
		slist_del(&list);
		free(elt);
	}

	return EXIT_SUCCESS;

	/* Error handling. */
failure:
	fputs("Error: cannot allocate memory. Exit...\n", stderr);
	if (!slist_is_empty(&list)) {
		/* List destruction with `slist_foreach_safe` and `slist_del`. */
		slist_foreach_safe(&list, curr, tmp) {
			elt = slist_elt(curr, pair_list, node);
			slist_del(&list);
			free(elt);
		}
	}
	return EXIT_FAILURE;
}
