/***********************************************************************
 * Copyright (c) 2013, Sylvain LAPERCHE                                *
 * All rights reserved.                                                *
 * License: BSD 3-Clause (http://opensource.org/licenses/BSD-3-Clause) *
 ***********************************************************************/
/*
 *    Filename: sort.c
 * Description: A simple example to illustrate how to sort a list.
 *     Created: 2013/08/26 14:19:03
 *      Author: Sylvain Laperche <sylvain.laperche@gmail.com>
 */
#include <stdio.h>
#include <stdlib.h>

#include "slist.h"

/* Size of the array. */
#define N 9

typedef struct {
	int i;
	int j;
	slist_node node;
} pair_list;

int cmp_by_i(slist_node* node1, slist_node* node2);

int main(void)
{
	int data[N][2] = { { 7, 1 }, { 5, 1 }, { 0, 1 },
			   { 3, 1 }, { 3, 2 }, { 0, 2 },
			   { 6, 1 }, { 9, 1 }, { 7, 2 } };
	slist list;
	slist_node* curr;
	slist_node* tmp;
	pair_list*  elt;
	size_t i;
	/* Initialize the list. */
	slist_init(&list);
	/* Populate the list. */
	for(i = 0; i < N; ++i) {
		elt = malloc(sizeof *elt);
		if (elt == NULL)
			goto failure;

		elt->i = data[i][0];
		elt->j = data[i][1];
		slist_append(&list, &(elt->node));
	}
	/* Initial state. */
	puts("List content (before sorting):");
	slist_foreach_elt(&list, elt, pair_list, node)
		printf("i = %u | j = %u\n", elt->i, elt->j);
	/* Sorting. */
	slist_sort(&list, cmp_by_i);
	/* Final state. */
	puts("\nList content (after sorting):");
	slist_foreach_elt(&list, elt, pair_list, node)
		printf("i = %u | j = %u\n", elt->i, elt->j);
	/* Memory deallocation. */
	slist_foreach_elt_safe(&list, elt, tmp, pair_list, node) {
		slist_del(&list);
		free(elt);
	}

	return EXIT_SUCCESS;

	/* Error handling. */
failure:
	fputs("Error: cannot allocate memory. Exit...\n", stderr);
	if (!slist_is_empty(&list)) {
		/* List destruction with `slist_foreach_safe` and `slist_del`. */
		slist_foreach_safe(&list, curr, tmp) {
			elt = slist_elt(curr, pair_list, node);
			slist_del(&list);
			free(elt);
		}
	}
	return EXIT_FAILURE;
}

int cmp_by_i(slist_node* node1, slist_node* node2)
{
	pair_list* p1 = slist_elt(node1, pair_list, node);
	pair_list* p2 = slist_elt(node2, pair_list, node);

	return p1->i - p2->i;
}
